/*
 * Copyright (c) 1997, by Sun  Microsystems, Inc.
 * All rights reserved
 *
 * Derived from dv version 1.31, last changed 97/06/05
 */

#ifndef	_SYS_CMS_API_H
#define	_SYS_CMS_API_H

#pragma ident	"@(#)cms_api.h	1.6	98/11/11 SMI"

#include <time.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 * ******************************************************************
 * The CMS API Defintion Header File
 *
 * This file contains the CMS API functions declarations for the CMS
 * API library "libcms.so".  These functions are to be used by
 * clients to access the managed objects within the CMS. To use these
 * functions, the client must create and manipulate the CMS API
 * Objects defined within this header file.
 *
 * The methods used to manipulate these objects are defined by the
 * structure of the objects. The definitions of enummerated types
 * used by the CMS API are also defined within this header file.
 * ******************************************************************
 */

/* CMS API Constraints */

#define	CMS_maximum_value_length	64
#define	CMS_maximum_number_instances	32
#define	CMS_maximum_number_attributes	32

/* Type definitions */

typedef enum cms_attribute_type {
	CMS_attribute_type_invalid,
	CMS_attribute_type_string
} CMS_attr_type;

typedef enum cms_attribute_access {
	CMS_attribute_access_invalid,
	CMS_attribute_access_system,
	CMS_attribute_access_user
} CMS_attr_access;

typedef enum cms_bool {
	CMS_boolean_invalid = -1,
	CMS_boolean_false = 0,
	CMS_boolean_true = 1
} CMS_boolean;

typedef enum cms_scope {
	CMS_scope_invalid,
	CMS_scope_none,
	CMS_scope_me,
	CMS_scope_my_ancestors,
	CMS_scope_my_children,
	CMS_scope_my_classes,
	CMS_scope_my_definition,
	CMS_scope_my_descendents,
	CMS_scope_my_instances,
	CMS_scope_my_parents
} CMS_scope;

typedef enum cms_filter {
	CMS_filter_invalid,
	CMS_filter_none,
	CMS_filter_module_faulty,
	CMS_filter_module_present,
	CMS_filter_name_equal,
	CMS_filter_value_equal,
	CMS_filter_value_not_equal
} CMS_filter;

typedef enum cms_sf_defined {
	CMS_sf_invalid,
	CMS_sf_none,
	CMS_sf_modules_faulty,
	CMS_sf_modules_present,
	CMS_sf_my_required_condition
} CMS_sf_defined;

typedef enum cms_error_class {
	CMS_error_class_invalid,
	CMS_error_class_no_error,
	CMS_error_class_hard,
	CMS_error_class_internal,
	CMS_error_class_soft,
	CMS_error_class_unrecoverable,
	CMS_error_class_warning
} CMS_error_class;

typedef enum cms_error_code {
	CMS_error_code_success,
	CMS_error_code_failure,
	CMS_error_code_abdication,
	CMS_error_code_access_permission,
	CMS_error_code_connection_lost,
	CMS_error_code_efd_not_registered,
	CMS_error_code_index_out_of_range,
	CMS_error_code_illegal_access,
	CMS_error_code_internal_error,
	CMS_error_code_invalid,
	CMS_error_code_invalid_attribute,
	CMS_error_code_invalid_efd,
	CMS_error_code_invalid_filter,
	CMS_error_code_invalid_valu,
	CMS_error_code_no_error,
	CMS_error_code_no_object,
	CMS_error_code_null_parameter,
	CMS_error_code_rules_file,
	CMS_error_code_rule_violation,
	CMS_error_code_session_registered,
	CMS_error_code_session_not_registered,
	CMS_error_code_socket_error,
	CMS_error_code_string_too_long,
	CMS_error_code_too_many_instances,
	CMS_error_code_too_many_attributes,
	CMS_error_code_unknown_attribute,
	CMS_error_code_unknown_class,
	CMS_error_code_unknown_module,
	CMS_error_code_wrong_object_class
} CMS_error_code;

#define	CMS_success				  CMS_error_code_success
#define	CMS_failure				  CMS_error_code_failure

typedef enum cms_error_operation {
	CMS_error_operation_invalid,
	CMS_error_operation_no_error,
	CMS_error_operation_access,
	CMS_error_operation_cancel,
	CMS_error_operation_close,
	CMS_error_operation_create,
	CMS_error_operation_delete,
	CMS_error_operation_get,
	CMS_error_operation_open,
	CMS_error_operation_request,
	CMS_error_operation_set
} CMS_error_operation;

/* Forward Declarations */

typedef struct CMS_error_struct *CMS_error_object;

typedef struct CMS_notification_record_struct
*CMS_notification_record_object;

typedef
    void (*CMS_notification_handler) (CMS_notification_record_object record);


/*
 * ************************************************************************
 * 			DEFINITION OF THE ATTRIBUTE OBJECT
 * ************************************************************************
 */

typedef struct CMS_attribute_struct *CMS_attribute_object;
struct CMS_attribute_struct {
	int private_identifier;
	CMS_error_object error;
	int (*destroy) (CMS_attribute_object me);
	CMS_attr_access(*getAccess) (CMS_attribute_object me);
	void *(*getDefaultValue) (CMS_attribute_object me);
	CMS_filter(*getFilter) (CMS_attribute_object me);
	char *(*getName) (CMS_attribute_object me);
	int (*getNumberPossibleValues)
	(CMS_attribute_object me);
	CMS_attr_type(*getType) (CMS_attribute_object me);
	void *(*getPastValue) (CMS_attribute_object me);
	void *(*getPossibleValue) (CMS_attribute_object me, int index);
	void *(*getValue) (CMS_attribute_object me);
	int (*setClassName) (CMS_attribute_object me, char *name);
	int (*setFilter) (CMS_attribute_object me, CMS_filter filter);
	int (*setInstanceIdentifier) (CMS_attribute_object me, char *ident);
	int (*setName) (CMS_attribute_object me, char *name);
	int (*setValue) (CMS_attribute_object me, void *value);
};

/*
 * ************************************************************************
 * 			DEFINITION OF THE ERROR OBJECT
 * ************************************************************************
 */

struct CMS_error_struct {
	int private_identifier;
	void *(*getAllMessages) (CMS_error_object me);
	char *(*getCheckErrorAtNumber) (CMS_error_object me, int index);
	char *(*getCheckWarningAtNumber) (CMS_error_object me, int index);
	CMS_error_code(*getCode) (CMS_error_object me);
	void *(*getMessage) (CMS_error_object me);
	int (*getNumberCheckErrors) (CMS_error_object me);
	int (*getNumberCheckWarnings) (CMS_error_object me);
	CMS_error_operation *(*getOperation) (CMS_error_object me);
	CMS_boolean(*isHard) (CMS_error_object me);
	CMS_boolean(*isSoft) (CMS_error_object me);
	CMS_boolean(*isUnrecoverable) (CMS_error_object me);
	CMS_boolean(*isWarning) (CMS_error_object me);
};

/*
 * ************************************************************************
 * 			DEFINITION OF THE EVENT OBJECT
 * ************************************************************************
 */

typedef struct CMS_event_struct *CMS_event_object;
struct CMS_event_struct {
	int private_identifier;
	CMS_error_object error;
	int (*destroy) (CMS_event_object me);
	int (*getEFDIdentifier) (CMS_event_object me, int *EFD_identifier);
	int (*setArg) (CMS_event_object me, void *arg);
	int (*setEventHandler) (CMS_event_object me,
	    CMS_notification_handler event_handler);
};

/*
 * ************************************************************************
 * 			DEFINITION OF THE EVENT RECORD OBJECT
 * ************************************************************************
 */

typedef struct CMS_ev_record_struct *CMS_event_record_object;
struct CMS_ev_record_struct {
	int private_identifier;
	CMS_error_object error;
	int (*destroy) (CMS_event_record_object me);
	void *(*getArg) (CMS_event_record_object me);
	int (*getAttribute) (CMS_event_record_object me,
	    CMS_attribute_object * attribute);
	int (*getEFDIdentifier) (CMS_event_record_object me, int *identifier);
	int (*getEventTimeInNanoseconds)
	(CMS_event_record_object me, long *event_time);
	int (*getEventTimeInSeconds) (CMS_event_record_object me,
	    time_t * event_time);
	int (*getInstanceIdentifier) (CMS_event_record_object me,
	    char *instanceid);
	void *(*getReasonString) (CMS_event_record_object me);
	CMS_boolean(*isStampedByCMS) (CMS_event_record_object me);
	CMS_boolean(*isStampedByLogger) (CMS_event_record_object me);
	CMS_boolean(*isAssociated) (CMS_event_record_object me);
};

/*
 * ************************************************************************
 * 			DEFINITION OF THE EXCEPTION OBJECT
 * ************************************************************************
 */

typedef struct CMS_exception_struct *CMS_exception_object;
struct CMS_exception_struct {
	int private_identifier;
	CMS_error_object error;
	int (*destroy) (CMS_exception_object me);
	int (*getEFDIdentifier) (CMS_exception_object me, int *EFD_identifier);
	int (*setArg) (CMS_exception_object me, void *arg);
	int (*setExceptionHandler) (CMS_exception_object me,
	    CMS_notification_handler exception_handler);
};

/*
 * ************************************************************************
 * 			DEFINITION OF THE EXCEPTION RECORD OBJECT
 * ************************************************************************
 */

typedef struct CMS_ex_record_struct *CMS_exception_record_object;
struct CMS_ex_record_struct {
	int private_identifier;
	CMS_error_object error;
	int (*destroy) (CMS_exception_record_object me);
	void *(*getArg) (CMS_exception_record_object me);
	int (*getEFDIdentifier) (CMS_exception_record_object me,
	    int *identifier);
	CMS_boolean(*isAbdication) (CMS_exception_record_object me);
	CMS_boolean(*isConnectionLost) (CMS_exception_record_object me);
	CMS_boolean(*isSocketError) (CMS_exception_record_object me);
};

/*
 * ************************************************************************
 * 			DEFINITION OF THE GENNERALISED OBJECT
 * ************************************************************************
 */

typedef struct CMS_object_struct *CMS_object;
struct CMS_object_struct {
	int private_identifier;
	CMS_error_object error;
};

/*
 * ************************************************************************
 * 			DEFINITION OF THE MANAGED OBJECT
 * ************************************************************************
 */

typedef struct CMS_managed_struct *CMS_managed_object;
struct CMS_managed_struct {
	int private_identifier;
};

/*
 * ************************************************************************
 * 			DEFINITION OF THE MODULE OBJECT
 * ************************************************************************
 */

typedef struct CMS_module_struct *CMS_module_object;
struct CMS_module_struct {
	int private_identifier;
	CMS_error_object error;
	int (*destroy) (CMS_module_object me);
	int (*getAttributeAtNumber) (CMS_module_object me,
	    int index,
	    CMS_attribute_object * attribute);
	int (*getAttributeWithName) (CMS_module_object me,
	    char *attribute_name, CMS_attribute_object * attribute);
	int (*getNumberOfAttributes) (CMS_module_object me,
	    int *number_attributes);
	int (*setInstanceIdentifier) (CMS_module_object me,
	    char *identifier);
	int (*setClassName) (CMS_module_object me,
	    char *class_name);
};

/*
 * ************************************************************************
 * 			DEFINITION OF THE NOTIFICATION OBJECT
 * ************************************************************************
 */

typedef struct CMS_notification_struct *CMS_notification_object;
struct CMS_notification_struct {
	int private_identifier;
};

/*
 * ************************************************************************
 * 			DEFINITION OF THE NOTIFICATION RECORD OBJECT
 * ************************************************************************
 */

struct CMS_notification_record_struct {
	int private_identifier;
};


/*
 * ************************************************************************
 * 			DEFINITION OF THE SCOPE AND FILTER OBJECT
 * ************************************************************************
 */

typedef struct CMS_scope_filter_struct *CMS_scope_filter_object;
struct CMS_scope_filter_struct {
	int private_identifier;
	CMS_error_object error;
	int (*addFilter) (CMS_scope_filter_object me,
	    CMS_attribute_object attribute);
	int (*destroy) (CMS_scope_filter_object me);
	    CMS_scope(*getScope) (CMS_scope_filter_object me);
	int (*resetFilters) (CMS_scope_filter_object me);
	int (*setScope) (CMS_scope_filter_object me, CMS_scope scope);
	int (*setPreDefined) (CMS_scope_filter_object me,
	    CMS_sf_defined pre_defined);
};

/*
 * ************************************************************************
 * 			DEFINITION OF THE SESSION OBJECT
 * ************************************************************************
 */

typedef struct CMS_session_struct *CMS_session_object;
struct CMS_session_struct {
	int private_identifier;
	CMS_error_object error;
	int (*destroy) (CMS_session_object me);
	int (*excludeMeFromRuleChecking) (CMS_session_object me);
	int (*includeMeInRuleChecking) (CMS_session_object me);
	int (*selectFullRuleChecking) (CMS_session_object me);
	int (*selectIgnoreRuleWarnings) (CMS_session_object me);
	int (*selectNoRuleChecking) (CMS_session_object me);
	int (*selectRuleCheckOnly) (CMS_session_object me);
	int (*setUncachedSession) (CMS_session_object me);
	int (*setRelativeSession) (CMS_session_object me);
	char *(*getOldConfigfilePath) (CMS_session_object me);
};


/*
 * ************************************************************************
 * 			DEFINITION OF THE SYSTEM OBJECT
 * ************************************************************************
 */

typedef struct CMS_system_struct *CMS_system_object;
struct CMS_system_struct {
	int private_identifier;
	CMS_error_object error;
	/* Public Methods */
	int (*destroy) (CMS_system_object me);
	int (*getClassName) (CMS_system_object me, int index,
	    char *class_name);
	int (*getInstanceIdentifier) (CMS_system_object me, int index,
	    char *instance_identifier);
	int (*getNumberOfClasses) (CMS_system_object me, int *number_classes);
	int (*getNumberOfModules) (CMS_system_object me, int *number_modules);
};

/*
 * ************************************************************************
 * 			OBJECT CONSTRUCTORS
 * ************************************************************************
 */

extern int cms_create_attribute(CMS_attribute_object * attribute);
extern int cms_create_event(CMS_event_object * event);
extern int cms_create_event_record(CMS_event_record_object * ev_record);
extern int cms_create_exception(CMS_exception_object * exception);
extern int cms_create_exception_record(CMS_exception_record_object * ex_record);
extern int cms_create_module(CMS_module_object * module);
extern int cms_create_scope_filter(CMS_scope_filter_object * scope_filter);
extern int cms_create_session(CMS_session_object * session);
extern int cms_create_system(CMS_system_object * system);

/*
 * ************************************************************************
 * 			THE CMS API COMMANDS
 * ************************************************************************
 */

extern int cms_open(CMS_session_object session);

extern int cms_close(CMS_session_object session);

extern int cms_get(CMS_session_object session,
    CMS_managed_object managed_object,
    CMS_scope_filter_object scope_filter);

extern int cms_set(CMS_session_object session,
    CMS_managed_object managed_object,
    CMS_scope_filter_object scope_filter);

extern int cms_has_event_occurred(CMS_session_object session,
    CMS_boolean * event);

extern int cms_request_notifications(CMS_session_object session,
    CMS_notification_object notification,
    CMS_scope_filter_object scope_filter);

extern int cms_cancel_notifications(CMS_session_object session,
    CMS_notification_object notifications,
    CMS_scope_filter_object scope_filter);

extern char *cms_get_error_code_string(CMS_error_code error_code);

#ifdef __cplusplus
}
#endif

#endif	/* _SYS_CMS_API_H */
